﻿#region
##############################################################################################
# Script zum Export von VMs unter Hyper-V mit Windows Server 2016 oder Windows 10 Pro        #
# erstellt von Jan Kappen - j.kappen@rachfahl.de                                             #
# Version 0.4                                                                                #
# 17. Dezember 2016                                                                          #
# Diese Script wird bereitgestellt wie es ist, ohne jegliche Garantie. Der Einsatz           #
# erfolgt auf eigene Gefahr. Es wird jegliche Haftung ausgeschlossen.                        #
# Wer dem Autor etwas Gutes tun möchte, er trinkt gern ein kaltes Corona :)                  #
#                                                                                            #
# Dieses Script beinhaltet keine Hilfe. RTFM.                                                #
#                                                                                            #
# www.hyper-v-server.de | www.rachfahl.de                                                    #
#                                                                                            #
##############################################################################################
#endregion

Param(
	[string] $VM ="",
	[string] $Exportpfad = "",
	[string] $Logpfad = "",
	[switch] $Speichern,
    [switch] $ProductionCheckpoint,
    [switch] $Herunterfahren,
    [switch] $statusmail
)

function Mail
{
    Write-Host -ForegroundColor Red (Get-Date) "Die Mail-Konfiguration muss angepasst werden, sonst ist kein Mailversand möglich!"
    # $mail = @{
    #    SmtpServer = 'mailer.domain.loc'
    #    Port = 25
    #    From = 'backupskript@rachfahl.de'
    #    To = 'empfaenger@domain.loc'
    #    Subject = "'Script-Sicherung der VM' $VM"
    #    Body = "'Anbei das Log der Export-Sicherung von VM' $VM"
    #    Attachments = "$Logdatei"
    # }
    # Send-MailMessage @mail
}

#region Logdatei
$LogDateiDatum = Get-Date -Format yyyy-MM-dd
if (!$Logpfad) {
                    $LogPfadVorhanden = Test-Path ${env:homedrive}\windows\Logs\HyperVExport\
                    if ($LogPfadVorhanden -eq $False) { new-item ${env:homedrive}\windows\Logs\HyperVExport\ -itemtype directory }
                    $Logdatei = "${env:homedrive}\windows\Logs\HyperVExport\$LogDateiDatum.log" 
                }
                else 
                { $Logdatei = "$Logpfad\$LogDateiDatum.log" }
#endregion

#region Abfrage auf benötigte Parameter
if (!$VM) { 
              Write-Host -ForegroundColor Red (Get-Date) "Parameter -VM muss vorhanden sein und einen Namen enthalten. Abbruch!"
              $temp1 = (Get-Date); $temp2 = "Parameter -VM muss vorhanden sein und einen Namen enthalten. Abbruch!"; "$temp1 - $temp2" | Out-File $Logdatei -Append
              exit 
          }
if (!$Exportpfad) { 
                      Write-Host -ForegroundColor Red (Get-Date)  "Parameter -RemotePfad muss vorhanden sein und einen Pfad enthalten. Abbruch!"
                      $temp1 = (Get-Date); $temp2 = "Parameter -RemotePfad muss vorhanden sein und einen Pfad enthalten. Abbruch!"; "$temp1 - $temp2" | Out-File $Logdatei -Append
                      exit
                  }
#endregion

#region Export der VM
if ($Speichern -match "true")
{
    Write-Host -ForegroundColor Green (Get-Date) "VM wird gespeichert"
    $temp1 = (Get-Date); $temp2 = "VM wird gespeichert"; "$temp1 - $temp2" | Out-File $Logdatei -Append
    Save-VM -Name $VM
    Write-Host -ForegroundColor Green (Get-Date) "Export der VM"
    $temp1 = (Get-Date); $temp2 = "Export der VM $VM"; "$temp1 - $temp2" | Out-File $Logdatei -Append
    $ZielPfadVorhanden = Test-Path $Exportpfad\$VM
    if 
        ($ZielPfadVorhanden -eq $False) { Export-VM -Name $VM -Path $Exportpfad }
    else 
        { Remove-Item -Recurse -Force $Exportpfad\$VM; Export-VM -Name $VM -Path $Exportpfad }
    Write-Host -ForegroundColor Green (Get-Date) "Export abgeschlossen, VM wird wieder eingeschaltet"
    $temp1 = (Get-Date); $temp2 = "Export abgeschlossen, VM wird wieder eingeschaltet"; "$temp1 - $temp2" | Out-File $Logdatei -Append
    Start-VM -Name $VM
    if ($statusmail -match "true")
        { Mail; exit }
    else
        { exit }
}

if ($Herunterfahren -match "true")
{
    $ShutdownStatus = get-vm -Name $VM | Get-VMIntegrationService | where { $_.Name -EQ "Shutdown" -or $_.Name -EQ "Herunterfahren" }
    if ($ShutdownStatus.Enabled -eq "True")
    {
        Write-Host -ForegroundColor Green (Get-Date) "Export scheint möglich zu sein, VM wird heruntergefahren"
        $temp1 = (Get-Date); $temp2 = "Export scheint möglich zu sein, VM wird heruntergefahren"; "$temp1 - $temp2" | Out-File $Logdatei -Append
        Stop-VM -Name $VM -Force
        Write-Host -ForegroundColor Green (Get-Date) "Export der VM"
        $temp1 = (Get-Date); $temp2 = "Export der VM $VM"; "$temp1 - $temp2" | Out-File $Logdatei -Append
        $ZielPfadVorhanden = Test-Path $Exportpfad\$VM
        if 
            ($ZielPfadVorhanden -eq $False) { Export-VM -Name $VM -Path $Exportpfad }
        else 
            { Remove-Item -Recurse -Force $Exportpfad\$VM; Export-VM -Name $VM -Path $Exportpfad }
        Write-Host -ForegroundColor Green (Get-Date) "VM wird wieder eingeschaltet"
        $temp1 = (Get-Date); $temp2 = "VM wird wieder eingeschaltet"; "$temp1 - $temp2" | Out-File $Logdatei -Append
        Start-VM -Name $VM
        if ($statusmail -match "true")
            { Mail; exit }
        else
            { exit }
    }
    else
    {
        Write-Host -ForegroundColor Red (Get-Date) "Export scheint nicht möglich zu sein, Vorgang wird abgebrochen!"
        $temp1 = (Get-Date); $temp2 = "Export scheint nicht möglich zu sein, Vorgang wird abgebrochen!"; "$temp1 - $temp2" | Out-File $Logdatei -Append
        exit
    }
}

if ($ProductionCheckpoint -match "true")
{
    $SnapshotName = "ExportScriptCheckpoint"
    Write-Host -ForegroundColor Green (Get-Date) "Checkpoint wird erstellt"
    $temp1 = (Get-Date); $temp2 = "Export der VM $VM"; "$temp1 - $temp2" | Out-File $Logdatei -Append

    $ZielPfadVorhanden = Test-Path $Exportpfad\$VM
    if 
        ($ZielPfadVorhanden -eq $False) 
        { Checkpoint-VM -Name $VM -SnapshotName $SnapshotName
          Export-VMSnapshot -VMName $VM -Name $SnapshotName -Path $Exportpfad
          Remove-VMSnapshot -VMName $VM -Name $SnapshotName 
        }
    else 
        { Remove-Item -Recurse -Force $Exportpfad\$VM
          Checkpoint-VM -Name $VM -SnapshotName $SnapshotName
          Export-VMSnapshot -VMName $VM -Name $SnapshotName -Path $Exportpfad
          Remove-VMSnapshot -VMName $VM -Name $SnapshotName 
        }
    Write-Host -ForegroundColor Green (Get-Date) "Export abgeschlossen"
    $temp1 = (Get-Date); $temp2 = "Export der VM $VM abgeschlossen"; "$temp1 - $temp2" | Out-File $Logdatei -Append
    if ($statusmail -match "true")
        { Mail; exit }
    else
        { exit }
}
else
{
    Write-Host -ForegroundColor Green (Get-Date) "VM wird online exportiert"
    Write-Host -ForegroundColor Green (Get-Date) "Export der VM"
    $temp1 = (Get-Date); $temp2 = "Export der VM $VM"; "$temp1 - $temp2" | Out-File $Logdatei -Append
    $ZielPfadVorhanden = Test-Path $Exportpfad\$VM
    if 
        ($ZielPfadVorhanden -eq $False) { Export-VM -Name $VM -Path $Exportpfad }
    else 
        { Remove-Item -Recurse -Force $Exportpfad\$VM; Export-VM -Name $VM -Path $Exportpfad }
    Write-Host -ForegroundColor Green (Get-Date) "Export der VM $VM abgeschlossen"
    $temp1 = (Get-Date); $temp2 = "Export der VM $VM abgeschlossen"; "$temp1 - $temp2" | Out-File $Logdatei -Append
    if ($statusmail -match "true")
        { Mail; exit }
    else
        { exit }
}
#endregion